from flask_sqlalchemy import SQLAlchemy, BaseQuery, Model

from pf_flask_db.cli.pf_flask_db_cli import init_flask_db_cli
from pf_flask_db.common.pf_flask_db_exception import PFFlaskDBException
from pf_flask_db.helper.pf_db_helper import PFDBHelper


class PFAppDatabase(SQLAlchemy):

    def __init__(self,
                 app=None, use_native_unicode=True, session_options=None,
                 metadata=None, query_class=BaseQuery, model_class=Model,
                 engine_options=None):
        if not query_class:
            query_class = PFDBHelper
        super().__init__(
            app=app, use_native_unicode=use_native_unicode,
            session_options=session_options, metadata=metadata,
            query_class=query_class, engine_options=engine_options, model_class=model_class
        )

    def init_app(self, app):
        super().init_app(app)
        if app:
            init_flask_db_cli(app, self)

    def run_sql(self, sql):
        try:
            connection = self.engine.connect()
            return connection.execute(sql)
        except Exception as e:
            raise PFFlaskDBException.ins().get_exception(e)


app_db = PFAppDatabase()



import click
from flask.cli import AppGroup, with_appcontext

pf_flask_db_cli = AppGroup("db-cli", help="PF Flask Database Manipulation")
_pf_flask_db_cli_db = None
_DB_CONFIG_NOT_FOUND = 'Database Configuration Not Found'


@pf_flask_db_cli.command("init", help="Initialize database tables")
@with_appcontext
def initialize():
    if _pf_flask_db_cli_db:
        _pf_flask_db_cli_db.create_all()
        click.secho('Successfully Initialize Database Tables', fg="blue", bold=True)
    else:
        click.secho(_DB_CONFIG_NOT_FOUND, fg="blue", bold=True)


@pf_flask_db_cli.command("destroy", help="Delete database tables")
@with_appcontext
def destroy():
    if _pf_flask_db_cli_db:
        _pf_flask_db_cli_db.drop_all()
        click.secho('Successfully Delete Database Tables', fg="green", bold=True)
    else:
        click.secho(_DB_CONFIG_NOT_FOUND, fg="blue", bold=True)


def init_flask_db_cli(app, db):
    global _pf_flask_db_cli_db
    _pf_flask_db_cli_db = db
    if app:
        app.cli.add_command(pf_flask_db_cli)



from pf_flask_db.common.pf_flask_db_exception import PFFlaskDBException
from pf_flask_db.helper.query_suggestion import Query
from pf_flask_db.pf_app_database import app_db


class PFModel(app_db.Model):
    __abstract__ = True
    query: Query
    _model_list = []

    def save(self):
        try:
            app_db.session.add(self)
            if self._model_list:
                app_db.session.add_all(self._model_list)
            app_db.session.commit()
        except Exception as e:
            raise PFFlaskDBException.ins().get_exception(e)

    def delete(self):
        try:
            app_db.session.delete(self)
            app_db.session.commit()
        except Exception as e:
            raise PFFlaskDBException.ins().get_exception(e)

    def add(self, model):
        self._model_list.append(model)
        return self

    def add_all(self, models: list):
        self._model_list = models
        return self



from flask import Flask
from pf_flask_db.pf_app_model import AppModel
from pf_flask_db.pf_app_database import app_db

app = Flask(__name__)

app.config["SQLALCHEMY_DATABASE_URI"] = "sqlite:///pf-flask-db.sqlite"
app.config["SQLALCHEMY_TRACK_MODIFICATIONS"] = False
app_db.init_app(app)


class Person(AppModel):
    first_name = app_db.Column(app_db.String(150), nullable=False)
    last_name = app_db.Column(app_db.String(150))
    email = app_db.Column(app_db.String(120), nullable=False)
    age = app_db.Column(app_db.Integer)
    income = app_db.Column(app_db.Float, default=0)


with app.app_context():
    app_db.create_all()


@app.route('/')
def bismillah():
    return "PF Flask DB Tutorial"


@app.route('/create')
def create():
    person = Person(first_name="First Name", last_name="Last Name", email="hmtmcse.com@gmail.com", age=22, income=500)
    person.save()
    response = "Data successfully Inserted"
    return response


@app.route('/update')
def update():
    person = Person.query.filter_by(id=1).first()
    if person:
        person.first_name = "FName Update"
        person.last_name = "LName Update"
        person.save()
    return "Data has been updated."


@app.route('/delete')
def delete():
    person = Person.query.filter_by(id=1).first()
    if person:
        person.delete()
    return "Record has been deleted"


@app.route('/list')
def list():
    response = ""
    persons = Person.query.all()
    for person in persons:
        response += person.first_name + " " + person.last_name + " " + person.email + "<br>"
    return response


@app.route('/create-bulk')
def create_bulk():
    total_record = 20
    person = Person(first_name="Flask", last_name="DB", email="flask-db@email.loc")
    for index in range(total_record):
        person.add(
            Person(
                first_name="First Name " + str(index),
                last_name="Last Name " + str(index),
                email="email-" + str(index) + "@email.loc",
                age=1 * index,
                income=100 * index)
        )
    person.save()
    response = str(total_record) + " Records successfully Inserted"
    return response


@app.route('/pagination')
@app.route('/pagination/<int:per_page>')
@app.route('/pagination/<int:page>/<int:per_page>')
def pagination(page: int = 1, per_page: int = 5):
    response = {
        "page": 0,
        "pages": 0,
        "per_page": 0,
        "totalItem": 0,
        "items": [],
    }
    persons = Person.query.paginate(page, per_page, error_out=False)
    if persons:
        response["page"] = persons.page
        response["pages"] = persons.pages
        response["per_page"] = persons.per_page
        response["totalItem"] = persons.total
        for person in persons.items:
            response["items"].append(person.first_name + " " + person.last_name + " " + person.email)
    return response


if __name__ == '__main__':
    app.run(debug=True)



from sqlalchemy.exc import IntegrityError
from pf_py_common.pf_exception import PFException


class PFFlaskDBException:
    EXCEPTION_TYPE = "FLASK_DB"

    def parse_integrity_error(self, exception: IntegrityError):
        try:
            if exception.orig and exception.orig.args and exception.orig.args[1]:
                return str(exception.orig.args[1])
        except:
            return "Integrity Error"
        return str(exception.orig)

    def get_exception(self, exception: Exception):
        if isinstance(exception, IntegrityError):
            return PFException(self.parse_integrity_error(exception), exception_type=self.EXCEPTION_TYPE)
        return PFException(str(exception), exception_type=self.EXCEPTION_TYPE)

    @staticmethod
    def ins():
        return PFFlaskDBException()



from pf_py_common.py_common import PyCommon
from pf_flask_db.pf_app_database import app_db
from pf_flask_db.helper.pf_db_model import PFModel


class BaseModel(PFModel):
    __abstract__ = True


class AppModel(BaseModel):
    __abstract__ = True
    id = app_db.Column("id", app_db.BigInteger, primary_key=True)
    created = app_db.Column("created", app_db.DateTime, default=app_db.func.now())
    updated = app_db.Column("updated", app_db.DateTime, default=app_db.func.now(), onupdate=app_db.func.now())
    isDeleted = app_db.Column("is_deleted", app_db.Boolean, default=False)
    uuid = app_db.Column("uuid", app_db.String(50), default=PyCommon.uuid())



from setuptools import setup, find_packages
import os
import pathlib

CURRENT_DIR = pathlib.Path(__file__).parent
README = (CURRENT_DIR / "readme.md").read_text()

env = os.environ.get('dev')


def get_dependencies():
    dependency = [
        'Flask',
        'Flask-SQLAlchemy'
    ]

    if env and env == "dev":
        return dependency

    return dependency + ["PF-PY-Common"]


setup(
    name='PF-Flask-DB',
    version='1.0.0',
    url='https://github.com/problemfighter/pf-flask-db',
    license='Apache 2.0',
    author='Problem Fighter',
    author_email='problemfighter.com@gmail.com',
    description='Flask Database Manager by Problem Fighter Library',
    long_description=README,
    long_description_content_type='text/markdown',
    packages=find_packages(),
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    install_requires=get_dependencies(),
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
    ]
)



from flask import Flask
from pf_flask_db.pf_app_model import AppModel
from pf_flask_db.pf_app_database import app_db

app = Flask(__name__)

app.config["SQLALCHEMY_DATABASE_URI"] = "sqlite:///pf-flask-db-raw.sqlite"
app.config["SQLALCHEMY_TRACK_MODIFICATIONS"] = False
app_db.init_app(app)


class Person(AppModel):
    first_name = app_db.Column(app_db.String(150), nullable=False)
    last_name = app_db.Column(app_db.String(150))
    email = app_db.Column(app_db.String(120), nullable=False)
    age = app_db.Column(app_db.Integer)
    income = app_db.Column(app_db.Float, default=0)


with app.app_context():
    app_db.create_all()


@app.route('/')
def bismillah():
    return "PF Flask DB Tutorial"


@app.route('/create')
def create():
    insert = "INSERT INTO person (first_name, last_name, email) VALUES ('Raw First Name', 'Raw Last Name', 'hmtmcse.com@gmail.com')"
    app_db.run_sql(insert)
    response = "Data successfully Inserted"
    return response


@app.route('/update')
def update():
    sql = "UPDATE person SET first_name = 'updated first name', last_name = 'updated last name' WHERE id = 1"
    persons = app_db.run_sql(sql)
    return "Data has been updated."


@app.route('/delete')
def delete():
    sql = "DELETE FROM person WHERE id = 1"
    persons = app_db.run_sql(sql)
    return "Record has been deleted"


@app.route('/list')
def list():
    response = ""
    sql = "SELECT * FROM person"
    persons = app_db.run_sql(sql)
    for person in persons:
        response += person.first_name + " " + person.last_name + " " + person.email + "<br>"
    return response


if __name__ == '__main__':
    app.run(debug=True)



from pf_flask_db.helper.pf_db_helper import PFDBHelper


class Query(PFDBHelper):

    def all(self):
        pass

    def first(self):
        pass

    def first_or_404(self, description=None):
        pass

    def paginate(self, page=None, per_page=None, error_out=True, max_per_page=None):
        pass

    def count(self):
        pass

    def get(self, ident):
        pass

    def get_or_404(self, ident, description=None):
        pass

    def with_entities(self, *entities):
        pass

    def filter(self, *criterion):
        return self

    def filter_by(self, **kwargs):
        return self

    def order_by(self, *clauses):
        pass

    def group_by(self, *clauses):
        pass

    def having(self, criterion):
        pass

    def join(self, target, *props, **kwargs):
        pass

    def limit(self, limit):
        pass

    def offset(self, offset):
        pass

    def from_statement(self, statement):
        pass

    def one_or_none(self):
        pass

    def one(self):
        pass

    def delete(self, synchronize_session="evaluate"):
        pass

    def update(self, values, synchronize_session="evaluate", update_args=None):
        pass



from flask import Flask
from pf_flask_db.pf_app_model import AppModel
from pf_flask_db.pf_app_database import app_db

app = Flask(__name__)

app.config["SQLALCHEMY_DATABASE_URI"] = "sqlite:///pf-flask-db-cli.sqlite"
app.config["SQLALCHEMY_TRACK_MODIFICATIONS"] = False
app_db.init_app(app)


class Person(AppModel):
    id = app_db.Column(app_db.Integer, primary_key=True)
    first_name = app_db.Column(app_db.String(150), nullable=False)
    last_name = app_db.Column(app_db.String(150))
    email = app_db.Column(app_db.String(120), nullable=False)
    age = app_db.Column(app_db.Integer)
    income = app_db.Column(app_db.Float, default=0)


@app.route('/')
def bismillah():
    return "PF Flask DB Tutorial"


@app.route('/create')
def create():
    person = Person(first_name="First Name", last_name="Last Name", email="hmtmcse.com@gmail.com", age=22, income=500)
    person.save()
    response = "Data successfully Inserted"
    return response


@app.route('/update')
def update():
    person = Person.query.filter_by(id=1).first()
    if person:
        person.first_name = "FName Update"
        person.last_name = "LName Update"
        person.save()
    return "Data has been updated."


if __name__ == '__main__':
    app.run(debug=True)
